DEFAULTS = {
    'managers': [],
    'lock_timer': -1,
    'auto_lock': False,
    'clipboard_timeout': 5,
    'copy_command': 'xclip',
    'danger_mode': False,
    'keybindings': {
        'type_all': 'alt+1',
        'type_user': 'alt+2',
        'type_pass': 'alt+3',
        'copy_totp': 'alt+t',
        'sync': 'alt+r'
    },
    'message': 'Default config loaded'
}
